@@ -5,18 +5,16 @@ use std::{
 };
 
 use harmony_rust_sdk::{
-    api::chat::event,
+    api::chat::{self, stream_event},
     client::{
         api::{
             auth::AuthStepResponse,
             chat::{
-                guild,
-                invite::InviteId,
-                message::{self, MessageExt, SendMessage, SendMessageSelfBuilder},
-                profile::{self, ProfileUpdate},
-                EventSource, UserId,
+                guild::JoinGuildRequest,
+                message::{MessageExt, SendMessage},
+                EventSource,
             },
-            harmonytypes::UserStatus,
+            profile::{GetProfileRequest, UpdateProfile, UserStatus},
         },
         error::ClientResult,
         Client,
@@ -80,19 +78,17 @@ async fn main() -> ClientResult<()> {
     }
 
     // Change our bots status to online and make sure its marked as a bot
-    profile::profile_update(
-        &client,
-        ProfileUpdate::default()
-            .new_status(UserStatus::OnlineUnspecified)
-            .new_is_bot(true),
-    )
-    .await?;
+    client
+        .call(
+            UpdateProfile::default()
+                .with_new_status(UserStatus::Online)
+                .with_new_is_bot(true),
+        )
+        .await?;
 
     // Join the guild if invite is specified
     let guild_id = if let Ok(invite) = guild_invite {
-        guild::join_guild(&client, InviteId::new(invite).unwrap())
-            .await?
-            .guild_id
+        client.call(JoinGuildRequest::new(invite)).await?.guild_id
     } else {
         tokio::fs::read_to_string(GUILD_ID_FILE)
             .await
@@ -115,7 +111,7 @@ async fn main() -> ClientResult<()> {
                 if DID_CTRLC.load(Ordering::Relaxed) {
                     return Ok(true);
                 }
-                if let event::Event::SentMessage(sent_message) = event {
+                if let chat::Event::Chat(stream_event::Event::SentMessage(sent_message)) = event {
                     if let Some(message) = sent_message.message {
                         let text_content = message.text().unwrap_or("<empty message>");
                         info!("got message: {}", text_content);
@@ -126,10 +122,8 @@ async fn main() -> ClientResult<()> {
                             if let Some(cmd) = parts.next() {
                                 let reply = match cmd {
                                     "ping" => {
-                                        let created_at = {
-                                            let tmp = message.created_at.unwrap_or_default();
-                                            Duration::new(tmp.seconds as u64, tmp.nanos as u32)
-                                        };
+                                        let created_at =
+                                            { Duration::from_secs(message.created_at) };
                                         let arrive_duration = (UNIX_EPOCH.elapsed().unwrap()
                                             - created_at)
                                             .as_secs_f32();
@@ -137,13 +131,17 @@ async fn main() -> ClientResult<()> {
                                         format!("Pong! Took {} secs.", arrive_duration)
                                     }
                                     "hello" => {
-                                        let user_profile = profile::get_user(
-                                            &client,
-                                            UserId::new(message.author_id),
-                                        )
-                                        .await?;
+                                        let user_profile = client
+                                            .call(GetProfileRequest::new(message.author_id))
+                                            .await?;
 
-                                        format!("Hello, {}!", user_profile.user_name)
+                                        format!(
+                                            "Hello, {}!",
+                                            user_profile
+                                                .profile
+                                                .as_ref()
+                                                .map_or("unknown", |p| p.user_name.as_str())
+                                        )
                                     }
                                     "uptime" => {
                                         format!(
@@ -153,13 +151,13 @@ async fn main() -> ClientResult<()> {
                                     }
                                     _ => "No such command.".to_string(),
                                 };
-                                message::send_message(
-                                    &client,
-                                    SendMessage::new(guild_id, message.channel_id)
-                                        .in_reply_to(message.message_id)
-                                        .text(reply),
-                                )
-                                .await?;
+                                client
+                                    .call(
+                                        SendMessage::new(guild_id, sent_message.channel_id)
+                                            .with_in_reply_to(sent_message.message_id)
+                                            .text(reply),
+                                    )
+                                    .await?;
                             }
                         }
                     }
@@ -170,11 +168,9 @@ async fn main() -> ClientResult<()> {
         .await?;
 
     // Change our bots status back to offline
-    profile::profile_update(
-        &client,
-        ProfileUpdate::default().new_status(UserStatus::Offline),
-    )
-    .await?;
+    client
+        .call(UpdateProfile::default().with_new_status(UserStatus::OfflineUnspecified))
+        .await?;
 
     Ok(())
 }
