@@ -2,18 +2,12 @@
 use std::sync::atomic::{AtomicBool, Ordering};
 
 use harmony_rust_sdk::{
-    api::chat::event,
+    api::chat::{self, stream_event, JoinGuildRequest},
     client::{
         api::{
             auth::AuthStepResponse,
-            chat::{
-                guild,
-                invite::InviteId,
-                message::{self, SendMessage, SendMessageSelfBuilder},
-                profile::{self, ProfileUpdate},
-                EventSource,
-            },
-            harmonytypes::UserStatus,
+            chat::{message::SendMessage, EventSource},
+            profile::{UpdateProfile, UserStatus},
         },
         error::ClientResult,
         Client,
@@ -77,19 +71,17 @@ async fn main() -> ClientResult<()> {
     }
 
     // Change our bots status to online and make sure its marked as a bot
-    profile::profile_update(
-        &client,
-        ProfileUpdate::default()
-            .new_status(UserStatus::OnlineUnspecified)
-            .new_is_bot(true),
-    )
-    .await?;
+    client
+        .call(
+            UpdateProfile::default()
+                .with_new_status(UserStatus::Online)
+                .with_new_is_bot(true),
+        )
+        .await?;
 
     // Join the guild if invite is specified
     let guild_id = if let Ok(invite) = guild_invite {
-        guild::join_guild(&client, InviteId::new(invite).unwrap())
-            .await?
-            .guild_id
+        client.call(JoinGuildRequest::new(invite)).await?.guild_id
     } else {
         tokio::fs::read_to_string(GUILD_ID_FILE)
             .await
@@ -113,22 +105,25 @@ async fn main() -> ClientResult<()> {
                 if DID_CTRLC.load(Ordering::Relaxed) {
                     return Ok(true);
                 }
-                if let event::Event::SentMessage(sent_message) = event {
+                if let chat::Event::Chat(stream_event::Event::SentMessage(sent_message)) = event {
                     if let Some(message) = sent_message.message {
                         // Dont sent message if we sent it
                         if message.author_id != self_id {
-                            info!("Echoing message: {}", message.message_id);
+                            info!("Echoing message: {}", sent_message.message_id);
 
-                            let mut send_message = SendMessage::new(guild_id, message.channel_id)
-                                .in_reply_to(message.in_reply_to)
-                                .overrides(message.overrides)
-                                .metadata(message.metadata);
+                            let mut send_message =
+                                SendMessage::new(guild_id, sent_message.channel_id)
+                                    .with_overrides(message.overrides)
+                                    .with_metadata(message.metadata);
 
+                            if let Some(in_reply_to) = message.in_reply_to {
+                                send_message = send_message.with_in_reply_to(in_reply_to);
+                            }
                             if let Some(content) = message.content {
-                                send_message = send_message.content(content);
+                                send_message = send_message.with_content(content);
                             }
 
-                            message::send_message(&client, send_message).await?;
+                            client.call(send_message).await?;
                         }
                     }
                 }
@@ -138,11 +133,9 @@ async fn main() -> ClientResult<()> {
         .await?;
 
     // Change our bots status back to offline
-    profile::profile_update(
-        &client,
-        ProfileUpdate::default().new_status(UserStatus::Offline),
-    )
-    .await?;
+    client
+        .call(UpdateProfile::default().with_new_status(UserStatus::OfflineUnspecified))
+        .await?;
 
     Ok(())
 }
