@@ -2,18 +2,12 @@
 use std::sync::atomic::{AtomicBool, Ordering};
 
 use harmony_rust_sdk::{
-    api::chat::event,
+    api::chat::{self, stream_event, JoinGuildRequest},
     client::{
         api::{
             auth::AuthStepResponse,
-            chat::{
-                guild,
-                invite::InviteId,
-                message::MessageExt,
-                profile::{self, ProfileUpdate},
-                EventSource,
-            },
-            harmonytypes::UserStatus,
+            chat::{message::MessageExt, EventSource},
+            profile::{UpdateProfile, UserStatus},
         },
         error::ClientResult,
         Client,
@@ -77,19 +71,17 @@ async fn main() -> ClientResult<()> {
     }
 
     // Change our bots status to online and make sure its marked as a bot
-    profile::profile_update(
-        &client,
-        ProfileUpdate::default()
-            .new_status(UserStatus::OnlineUnspecified)
-            .new_is_bot(true),
-    )
-    .await?;
+    client
+        .call(
+            UpdateProfile::default()
+                .with_new_status(UserStatus::Online)
+                .with_new_is_bot(true),
+        )
+        .await?;
 
     // Join the guild if invite is specified
     let guild_id = if let Ok(invite) = guild_invite {
-        guild::join_guild(&client, InviteId::new(invite).unwrap())
-            .await?
-            .guild_id
+        client.call(JoinGuildRequest::new(invite)).await?.guild_id
     } else {
         tokio::fs::read_to_string(GUILD_ID_FILE)
             .await
@@ -110,14 +102,14 @@ async fn main() -> ClientResult<()> {
                 if DID_CTRLC.load(Ordering::Relaxed) {
                     return Ok(true);
                 }
-                if let event::Event::SentMessage(sent_message) = event {
+                if let chat::Event::Chat(stream_event::Event::SentMessage(sent_message)) = event {
                     if let Some(message) = sent_message.message {
                         info!("Received new message: {:?}", message);
                         println!(
                             "Received new message with ID {}, from guild {} in channel {} sent by {}:\n{}",
-                            message.message_id,
-                            message.guild_id,
-                            message.channel_id,
+                            sent_message.message_id,
+                            sent_message.guild_id,
+                            sent_message.channel_id,
                             message.author_id,
                             message.text().unwrap_or("<empty message>"),
                         );
@@ -129,11 +121,9 @@ async fn main() -> ClientResult<()> {
         .await?;
 
     // Change our bots status back to offline
-    profile::profile_update(
-        &client,
-        ProfileUpdate::default().new_status(UserStatus::Offline),
-    )
-    .await?;
+    client
+        .call(UpdateProfile::default().with_new_status(UserStatus::OfflineUnspecified))
+        .await?;
 
     Ok(())
 }
