@@ -5,9 +5,13 @@ use std::{
     str::FromStr,
 };
 
-use hrpc::url::{ParseError as UrlParseError, Url};
+use http::{
+    uri::{InvalidUri as UrlParseError, Parts, Scheme},
+    Uri,
+};
 
 /// Rest API common types.
+#[cfg(feature = "rest")]
 pub mod rest;
 
 /// Chat service API.
@@ -55,6 +59,27 @@ pub mod sync {
     pub use v1::*;
 }
 
+/// Profile service API.
+#[cfg(feature = "gen_profile")]
+pub mod profile;
+
+/// Emote service API.
+#[cfg(feature = "gen_emote")]
+pub mod emote {
+    pub mod v1 {
+        hrpc::include_proto!("protocol.emote.v1");
+    }
+    pub use v1::*;
+}
+/// Batch service API.
+#[cfg(feature = "gen_batch")]
+pub mod batch {
+    pub mod v1 {
+        hrpc::include_proto!("protocol.batch.v1");
+    }
+    pub use v1::*;
+}
+
 /// Some crates re-exported for user convenience.
 pub mod exports {
     pub use hrpc;
@@ -87,7 +112,7 @@ impl Display for HmcParseError {
 impl StdError for HmcParseError {}
 
 /// Errors that can occur when converting a possibly non-HMC string to a HMC URL.
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub enum HmcFromStrError {
     /// Returned if the string isn't a valid URL.
     UrlParse(UrlParseError),
@@ -118,7 +143,7 @@ impl StdError for HmcFromStrError {
 /// An example HMC looks like `hmc://chat.harmonyapp.io/403cb46c-49cf-4ae1-b876-f38eb26accb0`.
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub struct Hmc {
-    inner: Url,
+    inner: Uri,
 }
 
 impl Hmc {
@@ -158,7 +183,7 @@ impl Hmc {
     /// assert_eq!(hmc.server(), "example.org");
     /// ```
     pub fn server(&self) -> &str {
-        self.inner.host_str().unwrap()
+        self.inner.host().unwrap()
     }
 
     /// Gets the port of this HMC.
@@ -170,19 +195,7 @@ impl Hmc {
     /// assert_eq!(hmc.port(), 2289);
     /// ```
     pub fn port(&self) -> u16 {
-        self.inner.port().unwrap_or(2289)
-    }
-
-    /// Get the HMC URL as a string reference.
-    ///
-    /// # Example
-    /// ```
-    /// # use harmony_rust_sdk::api::Hmc;
-    /// let hmc = Hmc::new("example.org:2289", "403cb46c-49cf-4ae1-b876-f38eb26accb0").unwrap();
-    /// assert_eq!(hmc.as_str(), "hmc://example.org:2289/403cb46c-49cf-4ae1-b876-f38eb26accb0");
-    /// ```
-    pub fn as_str(&self) -> &str {
-        self.inner.as_str()
+        self.inner.port_u16().unwrap_or(2289)
     }
 }
 
@@ -194,13 +207,7 @@ impl Display for Hmc {
 
 impl From<Hmc> for String {
     fn from(hmc: Hmc) -> String {
-        hmc.inner.into()
-    }
-}
-
-impl From<Hmc> for Url {
-    fn from(hmc: Hmc) -> Self {
-        hmc.inner
+        hmc.inner.to_string()
     }
 }
 
@@ -208,16 +215,16 @@ impl FromStr for Hmc {
     type Err = HmcFromStrError;
 
     fn from_str(value: &str) -> Result<Self, Self::Err> {
-        let url: Url = value.parse().map_err(HmcFromStrError::UrlParse)?;
+        let url = value.parse::<Uri>().map_err(HmcFromStrError::UrlParse)?;
 
         Self::try_from(url).map_err(HmcFromStrError::HmcParse)
     }
 }
 
-impl TryFrom<Url> for Hmc {
+impl TryFrom<Uri> for Hmc {
     type Error = HmcParseError;
 
-    fn try_from(value: Url) -> Result<Self, Self::Error> {
+    fn try_from(value: Uri) -> Result<Self, Self::Error> {
         if value.host().is_none() {
             return Err(HmcParseError::NoServer);
         };
@@ -246,21 +253,21 @@ mod test {
 
     #[test]
     fn parse_valid_hmc() {
-        Hmc::try_from(VALID_HMC.parse::<Url>().unwrap()).unwrap();
+        Hmc::try_from(VALID_HMC.parse::<Uri>().unwrap()).unwrap();
     }
 
     #[test]
     #[should_panic(expected = "InvalidId")]
     fn parse_invalid_id_hmc() {
-        if let Err(e) = Hmc::try_from(INVALID_ID_HMC.parse::<Url>().unwrap()) {
+        if let Err(e) = Hmc::try_from(INVALID_ID_HMC.parse::<Uri>().unwrap()) {
             panic!("{:?}", e)
         }
     }
 
     #[test]
-    #[should_panic(expected = "RelativeUrlWithoutBase")]
+    #[should_panic(expected = "NoServer")]
     fn parse_no_server_hmc() {
-        if let Err(e) = Hmc::try_from(NO_SERVER_HMC.parse::<Url>().unwrap()) {
+        if let Err(e) = Hmc::try_from(NO_SERVER_HMC.parse::<Uri>().unwrap()) {
             panic!("{:?}", e)
         }
     }
@@ -268,8 +275,94 @@ mod test {
     #[test]
     #[should_panic(expected = "NoId")]
     fn parse_no_id_hmc() {
-        if let Err(e) = Hmc::try_from(NO_ID_HMC.parse::<Url>().unwrap()) {
+        if let Err(e) = Hmc::try_from(NO_ID_HMC.parse::<Uri>().unwrap()) {
             panic!("{:?}", e)
         }
     }
 }
+
+/// An homeserver identifier containg a domain and a port.
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash)]
+pub struct HomeserverIdentifier {
+    domain: String,
+    port: u16,
+}
+
+impl HomeserverIdentifier {
+    /// Turn the identifier into the server URL.
+    pub fn to_url(&self) -> Uri {
+        let mut parts = Parts::default();
+        parts.scheme = Some(Scheme::from_str("https").unwrap());
+        parts.authority = Some(format!("{}:{}", self.domain, self.port).parse().unwrap());
+        Uri::from_parts(parts).unwrap()
+    }
+}
+
+/// Error thrown when parsing a homeserver ID from a string.
+#[derive(Debug, PartialEq, Eq)]
+pub enum HomeserverIdParseError {
+    /// Port wasn't a `u16`.
+    InvalidPort,
+    /// Port was missing.
+    MissingPort,
+    /// Domain was missing.
+    MissingDomain,
+    /// Homeserver ID was malformed.
+    Malformed,
+}
+
+impl Display for HomeserverIdParseError {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        let msg = match self {
+            Self::InvalidPort => "port is invalid (not an u16)",
+            Self::MissingPort => "port is missing",
+            Self::MissingDomain => "domain is missing",
+            Self::Malformed => "id is malformed",
+        };
+        f.write_str(msg)
+    }
+}
+
+impl StdError for HomeserverIdParseError {}
+
+impl FromStr for HomeserverIdentifier {
+    type Err = HomeserverIdParseError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        let mut split = s.split(':');
+
+        let domain = split.next().ok_or(HomeserverIdParseError::MissingDomain)?;
+        if domain.parse::<http::uri::Authority>().is_err() {
+            return Err(HomeserverIdParseError::Malformed);
+        }
+        let port_raw = split.next().ok_or(HomeserverIdParseError::MissingPort)?;
+
+        let port: u16 = port_raw
+            .parse()
+            .map_err(|_| HomeserverIdParseError::InvalidPort)?;
+
+        Ok(Self {
+            domain: domain.to_string(),
+            port,
+        })
+    }
+}
+
+/// A trait to facilitate easy request execution.
+pub trait Endpoint {
+    /// The response type of this endpoint request.
+    type Response;
+
+    /// The endpoint path of this request.
+    const ENDPOINT_PATH: &'static str;
+
+    #[cfg(feature = "_client_common")]
+    /// Execute the request using the provided client.
+    fn call_with(
+        self,
+        client: &crate::client::Client,
+    ) -> hrpc::exports::futures_util::future::LocalBoxFuture<
+        '_,
+        crate::client::error::ClientResult<hrpc::Response<Self::Response>>,
+    >;
+}
