@@ -3,8 +3,8 @@ use std::{convert::TryInto, error::Error as StdError, str::FromStr};
 use super::Hmc;
 use derive_more::{Display, From, Into, IntoIterator};
 use derive_new::new;
-use hrpc::url::Url;
-use http::HeaderValue;
+use http::{HeaderValue, Uri};
+use serde::{Deserialize, Serialize};
 
 /// Kind of the file downloaded.
 #[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
@@ -21,18 +21,7 @@ pub enum FileId {
     /// A plain ID. When you use this for a request, the `Client`s homeserver will be used.
     Id(String),
     /// An external URL. This MUST be an image according to the protocol.
-    External(Url),
-}
-
-impl FileId {
-    /// Get a string reference to this `FileId`.
-    pub fn as_str(&self) -> &str {
-        match self {
-            FileId::Hmc(hmc) => hmc.as_str(),
-            FileId::Id(id) => id.as_str(),
-            FileId::External(url) => url.as_str(),
-        }
-    }
+    External(Uri),
 }
 
 impl From<FileId> for String {
@@ -40,7 +29,7 @@ impl From<FileId> for String {
         match o {
             FileId::Hmc(hmc) => hmc.into(),
             FileId::Id(id) => id,
-            FileId::External(url) => url.into(),
+            FileId::External(url) => url.to_string(),
         }
     }
 }
@@ -59,18 +48,17 @@ impl FromStr for FileId {
         if s.is_empty() {
             Err(InvalidFileId)
         } else {
-            match s.parse::<Url>() {
+            match s.parse::<Uri>() {
                 Ok(url) => {
                     if let Ok(hmc) = url.clone().try_into() {
                         Ok(FileId::Hmc(hmc))
                     } else if !url.path().trim_start_matches('/').is_empty() {
                         Ok(FileId::External(url))
                     } else {
-                        Err(InvalidFileId)
+                        Ok(FileId::Id(s.to_owned()))
                     }
                 }
-                Err(hrpc::url::ParseError::RelativeUrlWithoutBase) => Ok(FileId::Id(s.to_owned())),
-                _ => Err(InvalidFileId),
+                _ => Ok(FileId::Id(s.to_owned())),
             }
         }
     }
@@ -126,3 +114,57 @@ pub fn extract_file_info_from_download_response<'a>(
 
     Ok((name, mimetype, kind))
 }
+
+/// Struct that implements `serde` `Deserialize` / `Serialize` and can be used for
+/// the [`/_harmony/about`](https://github.com/harmony-development/protocol/blob/main/rest/rest.md#get-_harmonyabout) endpoint.
+#[derive(Debug, Clone, Hash, PartialEq, Eq, Deserialize, Serialize)]
+pub struct About {
+    /// the Harmony server software being hosted.
+    #[serde(rename = "serverName")]
+    pub server_name: String,
+    /// the version of said Harmony server software.
+    pub version: String,
+    /// A description of why / who this server is hosted for.
+    #[serde(rename = "aboutServer")]
+    pub about_server: String,
+    /// "message of the day", can be used to put maintenance information.
+    #[serde(rename = "messageOfTheDay")]
+    pub message_of_the_day: String,
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use std::str::FromStr;
+
+    #[test]
+    fn parse_id() {
+        const ID: &str = "654624512343";
+        let file_id = FileId::from_str(ID).expect("file id parse");
+        assert!(matches!(file_id, FileId::Id(_)));
+        assert_eq!(ID.to_string(), file_id.to_string());
+    }
+
+    #[test]
+    fn parse_hmc() {
+        const HMC: &str = "hmc://chat.harmonyapp.io/654624512343";
+        let file_id = FileId::from_str(HMC).expect("file id parse");
+        assert!(matches!(file_id, FileId::Hmc(_)));
+        assert_eq!(HMC.to_string(), file_id.to_string());
+    }
+
+    #[test]
+    fn parse_uri() {
+        const URI: &str = "https://media.discordapp.net/attachments/330412938277945345/801119250269339728/unknown.png";
+        let file_id = FileId::from_str(URI).expect("file id parse");
+        assert!(matches!(file_id, FileId::External(_)));
+        assert_eq!(URI.to_string(), file_id.to_string());
+    }
+
+    #[test]
+    #[should_panic(expected = "InvalidFileId")]
+    fn parse_empty() {
+        const EMPTY: &str = "";
+        FileId::from_str(EMPTY).expect("file id parse");
+    }
+}
