@@ -1,6 +1,7 @@
 use super::*;
 use crate::client::error::ClientError;
 
+use http::Uri;
 use prost::bytes::Bytes;
 use reqwest::{multipart::*, Response};
 use serde::Deserialize;
@@ -17,29 +18,33 @@ pub async fn upload(
     content_type: String,
     data: Vec<u8>,
 ) -> ClientResult<Response> {
-    if !client.data.auth_status.lock().is_authenticated() {
+    let (status, bytes) = client.auth_status_lock().clone();
+    let token_bytes = if !status.is_authenticated() {
         return Err(ClientError::Unauthenticated);
-    }
+    } else {
+        bytes
+    };
 
-    // This unwrap is safe, since our client's homeserver url is valid, and the path we create is also checked at compile time.
-    let uri = client
-        .homeserver_url()
-        .join("/_harmony/media/upload")
-        .unwrap();
+    // [ref:upload_path_create]
+    let uri = format!("{}_harmony/media/upload", client.homeserver_url());
 
-    let form = Form::new().part("file", Part::bytes(data));
+    let form = Form::new().part(
+        "file",
+        Part::bytes(data)
+            .file_name(filename)
+            .mime_str(&content_type)?,
+    );
 
     let request = client
         .data
         .http
-        .post(uri.to_string().as_str())
+        .post(uri.as_str())
         .multipart(form)
         .header(http::header::AUTHORIZATION, unsafe {
             // This is safe on the assumption that servers will never send session tokens
             // with invalid-byte(s). If they do, they aren't respecting the protocol
-            http::HeaderValue::from_maybe_shared_unchecked(client.data.token_bytes.lock().clone())
+            http::HeaderValue::from_maybe_shared_unchecked(token_bytes)
         })
-        .query(&[("filename", &filename), ("contentType", &content_type)])
         .build()?;
     #[cfg(debug_assertions)]
     tracing::debug!("Sending HTTP request: {:?}", request);
@@ -51,14 +56,10 @@ pub async fn upload(
     response.error_for_status().map_err(Into::into)
 }
 
-/// Downloads a file using a file ID.
-///
-/// This endpoint does not require authentication.
-/// See [API documentation](https://github.com/harmony-development/protocol/blob/master/rest/rest.md#get-_harmonymediadownloadfile_id).
-pub async fn download(client: &Client, file_id: impl Into<FileId>) -> ClientResult<Response> {
+fn make_download_uri(homeserver_url: &Uri, file_id: impl Into<FileId>) -> String {
     const ENDPOINT: &str = "/_harmony/media/download/";
 
-    let uri = match file_id.into() {
+    match file_id.into() {
         FileId::Hmc(hmc) => format!(
             "https://{}:{}{}{}",
             hmc.server(),
@@ -68,21 +69,29 @@ pub async fn download(client: &Client, file_id: impl Into<FileId>) -> ClientResu
         )
         .parse()
         .unwrap(),
-        FileId::Id(id) => client
-            .homeserver_url()
-            .join(ENDPOINT)
-            .unwrap()
-            .join(&id)
-            .unwrap(),
-        FileId::External(uri) => client
-            .homeserver_url()
-            .join(ENDPOINT)
-            .unwrap()
-            .join(&urlencoding::encode(&uri.to_string()))
-            .unwrap(),
-    };
+        FileId::Id(id) => format!(
+            "{}{}{}",
+            homeserver_url,
+            ENDPOINT.trim_start_matches('/'),
+            id
+        ),
+        FileId::External(uri) => format!(
+            "{}{}{}",
+            homeserver_url,
+            ENDPOINT.trim_start_matches('/'),
+            urlencoding::encode(uri.to_string().as_str())
+        ),
+    }
+}
 
-    let request = client.data.http.get(uri.to_string().as_str()).build()?;
+/// Downloads a file using a file ID.
+///
+/// This endpoint does not require authentication.
+/// See [API documentation](https://github.com/harmony-development/protocol/blob/master/rest/rest.md#get-_harmonymediadownloadfile_id).
+pub async fn download(client: &Client, file_id: impl Into<FileId>) -> ClientResult<Response> {
+    let uri = make_download_uri(client.homeserver_url(), file_id);
+
+    let request = client.data.http.get(uri.as_str()).build()?;
     #[cfg(debug_assertions)]
     tracing::debug!("Sending HTTP request: {:?}", request);
 
@@ -179,36 +188,58 @@ pub async fn download_extract_file(
 #[cfg(test)]
 mod tests {
     use super::*;
-    use std::str::FromStr;
 
-    #[test]
-    fn parse_id() {
-        const ID: &str = "654624512343";
-        let file_id = FileId::from_str(ID).expect("file id parse");
-        assert!(matches!(file_id, FileId::Id(_)));
-        assert_eq!(ID.to_string(), file_id.to_string());
+    fn homeserver_uri() -> Uri {
+        Uri::from_static("https://chat.harmonyapp.io:2289")
     }
 
     #[test]
-    fn parse_hmc() {
-        const HMC: &str = "hmc://chat.harmonyapp.io/654624512343";
-        let file_id = FileId::from_str(HMC).expect("file id parse");
-        assert!(matches!(file_id, FileId::Hmc(_)));
-        assert_eq!(HMC.to_string(), file_id.to_string());
+    fn join_upload_path() {
+        // [tag:upload_path_create]
+        assert_eq!(
+            format!("{}_harmony/media/upload", homeserver_uri()),
+            "https://chat.harmonyapp.io:2289/_harmony/media/upload",
+        );
     }
 
     #[test]
-    fn parse_uri() {
-        const URI: &str = "https://media.discordapp.net/attachments/330412938277945345/801119250269339728/unknown.png";
-        let file_id = FileId::from_str(URI).expect("file id parse");
-        assert!(matches!(file_id, FileId::External(_)));
-        assert_eq!(URI.to_string(), file_id.to_string());
+    fn make_download_uri_with_id() {
+        let id = "test_id_asdfasdfa";
+        let uri = make_download_uri(&homeserver_uri(), FileId::Id(id.to_string()));
+        assert_eq!(
+            uri,
+            format!(
+                "https://chat.harmonyapp.io:2289/_harmony/media/download/{}",
+                id
+            ),
+        );
     }
 
     #[test]
-    #[should_panic(expected = "InvalidFileId")]
-    fn parse_empty() {
-        const EMPTY: &str = "";
-        FileId::from_str(EMPTY).expect("file id parse");
+    fn make_download_uri_with_hmc() {
+        let id = "test_hmc_asdfasdfa";
+        let hmc = Hmc::new("chat.harmonyapp.io:2289", id).expect("failed to create hmc");
+        let uri = make_download_uri(&homeserver_uri(), FileId::Hmc(hmc));
+        assert_eq!(
+            uri,
+            format!(
+                "https://chat.harmonyapp.io:2289/_harmony/media/download/{}",
+                id
+            ),
+        );
+    }
+
+    #[test]
+    fn make_download_uri_with_url() {
+        let ext_url_str = "https://cdn.discordapp.com/emojis/849034023640104980.png";
+        let ext_url = Uri::from_static(ext_url_str);
+        let uri = make_download_uri(&homeserver_uri(), FileId::External(ext_url));
+        assert_eq!(
+            uri,
+            format!(
+                "https://chat.harmonyapp.io:2289/_harmony/media/download/{}",
+                urlencoding::encode(ext_url_str)
+            ),
+        );
     }
 }
