@@ -12,20 +12,47 @@ pub mod exports {
     pub use reqwest;
 }
 
+use crate::api::{
+    auth::{BeginAuthRequest, NextStepResponse, StepBackResponse},
+    Endpoint,
+};
 use api::{auth::*, chat::EventSource, Hmc, HmcFromStrError};
 use error::*;
 
-use std::sync::Arc;
+use std::{
+    convert::TryFrom,
+    fmt::{self, Debug, Formatter},
+    sync::Arc,
+};
 
-use hrpc::{bytes::Bytes, url::Url};
+use hrpc::{
+    client::transport::{self as client_transport, TransportRequest, TransportResponse},
+    encode::encode_protobuf_message,
+    exports::{
+        bytes::{Bytes, BytesMut},
+        tower::Service,
+    },
+    Response,
+};
+use http::Uri;
 use parking_lot::{Mutex, MutexGuard};
 use reqwest::Client as HttpClient;
 use tokio::sync::Mutex as AsyncMutex;
 
-type AuthService = crate::api::auth::auth_service_client::AuthServiceClient;
-type ChatService = crate::api::chat::chat_service_client::ChatServiceClient;
+#[cfg(feature = "client_web")]
+type GenericClientTransport = client_transport::http::Wasm;
+#[cfg(all(feature = "client_native", not(feature = "client_web")))]
+type GenericClientTransport = client_transport::http::Hyper;
+type GenericClient = AddAuth<GenericClientTransport>;
+
+type AuthService = crate::api::auth::auth_service_client::AuthServiceClient<GenericClient>;
+type ChatService = crate::api::chat::chat_service_client::ChatServiceClient<GenericClient>;
 type MediaProxyService =
-    crate::api::mediaproxy::media_proxy_service_client::MediaProxyServiceClient;
+    crate::api::mediaproxy::media_proxy_service_client::MediaProxyServiceClient<GenericClient>;
+type ProfileService =
+    crate::api::profile::profile_service_client::ProfileServiceClient<GenericClient>;
+type EmoteService = crate::api::emote::emote_service_client::EmoteServiceClient<GenericClient>;
+type BatchService = crate::api::batch::batch_service_client::BatchServiceClient<GenericClient>;
 
 /// Represents an authentication state in which a [`Client`] can be.
 #[derive(Debug, Clone)]
@@ -70,12 +97,14 @@ impl AuthStatus {
 
 #[derive(Debug)]
 struct ClientData {
-    homeserver_url: Url,
-    token_bytes: Mutex<Bytes>,
-    auth_status: Mutex<AuthStatus>,
+    homeserver_url: Uri,
+    auth_status: Arc<Mutex<(AuthStatus, Bytes)>>,
     chat: AsyncMutex<ChatService>,
     auth: AsyncMutex<AuthService>,
     mediaproxy: AsyncMutex<MediaProxyService>,
+    profile: AsyncMutex<ProfileService>,
+    emote: AsyncMutex<EmoteService>,
+    batch: AsyncMutex<BatchService>,
     http: HttpClient,
 }
 
@@ -86,7 +115,7 @@ pub struct Client {
 }
 
 impl Client {
-    /// Create a new [`Client`] from a homeserver [`Url`] (URL) and an (optional) session.
+    /// Create a new [`Client`] from a homeserver [`Uri`] and an (optional) session.
     ///
     /// If port is not specified in the URL, this will add the default port `2289` to it.
     /// If scheme is not specified, this will assume the scheme is `https`.
@@ -100,10 +129,14 @@ impl Client {
     /// # Ok(())
     /// # }
     /// ```
-    pub async fn new(mut homeserver_url: Url, session: Option<Session>) -> ClientResult<Self> {
+    pub async fn new(mut homeserver_url: Uri, session: Option<Session>) -> ClientResult<Self> {
         // Add the default scheme if not specified
-        if !matches!(homeserver_url.scheme(), "http" | "https") {
-            homeserver_url.set_scheme("https").unwrap();
+        if !matches!(homeserver_url.scheme_str(), Some("http" | "https")) {
+            homeserver_url = {
+                let mut parts = homeserver_url.into_parts();
+                parts.scheme = Some("https".parse().unwrap());
+                Uri::from_parts(parts).unwrap()
+            };
         }
 
         let http = HttpClient::builder().build()?;
@@ -118,7 +151,11 @@ impl Client {
                 server: String,
             }
 
-            let url = homeserver_url.join("/_harmony/server").unwrap();
+            let url = {
+                let mut parts = homeserver_url.clone().into_parts();
+                parts.path_and_query = Some("/_harmony/server".parse().unwrap());
+                Uri::from_parts(parts).unwrap()
+            };
 
             if let Ok(response) = http
                 .get(&url.to_string())
@@ -127,16 +164,22 @@ impl Client {
                 .json::<Server>()
                 .await
             {
-                let host: Url = response.server.parse().unwrap();
+                let host: Uri = response.server.parse().unwrap();
                 homeserver_url = host;
             }
         };
 
         // Add the default port if not specified
         if homeserver_url.port().is_none() {
-            // These unwraps are safe since we use `Url` everywhere and we are sure that our `new_authority` is
-            // indeed a correct authority.
-            homeserver_url.set_port(Some(2289)).unwrap();
+            homeserver_url = {
+                let mut parts = homeserver_url.into_parts();
+                parts.authority = Some(
+                    format!("{}:2289", parts.authority.unwrap().as_str())
+                        .parse()
+                        .unwrap(),
+                );
+                Uri::from_parts(parts).unwrap()
+            }
         }
 
         #[cfg(debug_assertions)]
@@ -146,20 +189,36 @@ impl Client {
             session
         );
 
-        let auth = AuthService::new(http.clone(), homeserver_url.clone())?;
-        let chat = ChatService::new(http.clone(), homeserver_url.clone())?;
-        let mediaproxy = MediaProxyService::new(http.clone(), homeserver_url.clone())?;
-
         let session = session.map_or(AuthStatus::None, AuthStatus::Complete);
+        let token_bytes = session.session().map_or_else(Bytes::new, |s| {
+            Bytes::copy_from_slice(s.session_token.as_bytes())
+        });
+        let auth_status = Arc::new(Mutex::new((session, token_bytes)));
+
+        let transport = GenericClientTransport::new(homeserver_url.clone())
+            .map_err(|err| ClientError::Internal(InternalClientError::Transport(err)))?;
+        let transport = AddAuth {
+            inner: transport,
+            auth_status: auth_status.clone(),
+        };
+        let inner = hrpc::client::Client::new(transport);
+
+        let auth = AuthService::new_inner(inner.clone());
+        let chat = ChatService::new_inner(inner.clone());
+        let mediaproxy = MediaProxyService::new_inner(inner.clone());
+        let profile = ProfileService::new_inner(inner.clone());
+        let emote = EmoteService::new_inner(inner.clone());
+        let batch = BatchService::new_inner(inner);
+
         let data = ClientData {
             homeserver_url,
-            token_bytes: Mutex::new(session.session().map_or_else(Bytes::new, |s| {
-                Bytes::copy_from_slice(s.session_token.as_bytes())
-            })),
-            auth_status: Mutex::new(session),
+            auth_status,
             chat: AsyncMutex::new(chat),
             auth: AsyncMutex::new(auth),
             mediaproxy: AsyncMutex::new(mediaproxy),
+            profile: AsyncMutex::new(profile),
+            emote: AsyncMutex::new(emote),
+            batch: AsyncMutex::new(batch),
             http,
         };
 
@@ -180,79 +239,109 @@ impl Client {
     ) -> Result<(), ClientError>
     where
         Fut: std::future::Future<Output = ClientResult<bool>> + 'a,
-        Hndlr: Fn(&'a Client, crate::api::chat::event::Event) -> Fut,
+        Hndlr: Fn(&'a Client, crate::api::chat::Event) -> Fut,
     {
         let mut sock = self.subscribe_events(subs).await?;
         loop {
-            if let Some(res) = sock.get_event().await {
-                match res {
-                    Ok(event) => {
-                        if handler(&self, event).await? {
-                            return Ok(());
-                        }
+            match sock.get_event().await {
+                Ok(Some(ev)) => {
+                    if handler(self, ev).await? {
+                        return Ok(());
                     }
-                    Err(err) => tracing::error!("{}", err),
                 }
+                Err(err) => tracing::error!("{}", err),
+                _ => std::hint::spin_loop(),
             }
         }
     }
 
     #[inline(always)]
-    async fn chat(&self) -> tokio::sync::MutexGuard<'_, ChatService> {
+    /// Get a mutex guard to the chat service.
+    pub async fn chat(&self) -> tokio::sync::MutexGuard<'_, ChatService> {
         self.data.chat.lock().await
     }
 
     #[inline(always)]
-    async fn auth(&self) -> tokio::sync::MutexGuard<'_, AuthService> {
+    /// Get a mutex guard to the auth service.
+    pub async fn auth(&self) -> tokio::sync::MutexGuard<'_, AuthService> {
         self.data.auth.lock().await
     }
 
     #[inline(always)]
-    async fn mediaproxy(&self) -> tokio::sync::MutexGuard<'_, MediaProxyService> {
+    /// Get a mutex guard to the mediaproxy service.
+    pub async fn mediaproxy(&self) -> tokio::sync::MutexGuard<'_, MediaProxyService> {
         self.data.mediaproxy.lock().await
     }
 
     #[inline(always)]
-    fn auth_status_lock(&self) -> MutexGuard<AuthStatus> {
-        self.data.auth_status.lock()
+    /// Get a mutex guard to the profile service.
+    pub async fn profile(&self) -> tokio::sync::MutexGuard<'_, ProfileService> {
+        self.data.profile.lock().await
     }
 
     #[inline(always)]
-    pub(crate) async fn generic_api_fn<'a, Resp, Req, IntoReq, Handler, HnldrOut>(
-        &'a self,
-        handler: Handler,
-        request: IntoReq,
-    ) -> ClientResult<Resp>
+    /// Get a mutex guard to the emote service.
+    pub async fn emote(&self) -> tokio::sync::MutexGuard<'_, EmoteService> {
+        self.data.emote.lock().await
+    }
+
+    #[inline(always)]
+    /// Get a mutex guard to the batch service.
+    pub async fn batch(&self) -> tokio::sync::MutexGuard<'_, BatchService> {
+        self.data.batch.lock().await
+    }
+
+    /// Execute the given request.
+    pub async fn call<Req>(&self, request: Req) -> ClientResult<Req::Response>
     where
-        Req: std::fmt::Debug,
-        Resp: std::fmt::Debug,
-        IntoReq: Into<Req> + std::fmt::Debug,
-        Handler: FnOnce(&'a Client, hrpc::Request<Req>) -> HnldrOut,
-        HnldrOut: std::future::Future<Output = Result<Resp, InternalClientError>> + 'a,
+        Req: Endpoint,
+        Req::Response: prost::Message + Default,
     {
-        use hrpc::IntoRequest;
+        Ok(request.call_with(self).await?.into_message().await?)
+    }
 
-        let mut request = request.into().into_request();
-        if self.data.auth_status.lock().is_authenticated() {
-            request = request.header(
-                http::header::AUTHORIZATION,
-                // This is safe on the assumption that servers will never send session tokens
-                // with invalid-byte(s). If they do, they aren't respecting the protocol
-                unsafe {
-                    http::HeaderValue::from_maybe_shared_unchecked(
-                        self.data.token_bytes.lock().clone(),
-                    )
-                },
-            );
+    /// Execute the given request, but return a [`hrpc::Response`].
+    pub async fn call_response<Req: Endpoint>(
+        &self,
+        request: Req,
+    ) -> ClientResult<Response<Req::Response>> {
+        request.call_with(self).await
+    }
+
+    /// Execute the given requests a batch (same) request.
+    ///
+    /// Note that this does not support the convenience types defined in the [`api`] module.
+    /// You will need to convert them to the corresponding request type with `Request::from`.
+    pub async fn batch_call<Req>(
+        &self,
+        requests: Vec<Req>,
+    ) -> ClientResult<Vec<<Req as Endpoint>::Response>>
+    where
+        Req: Endpoint + prost::Message,
+        <Req as Endpoint>::Response: prost::Message + Default,
+    {
+        use prost::Message;
+
+        let encoded = requests
+            .iter()
+            .map(encode_protobuf_message)
+            .map(BytesMut::freeze);
+        let batch_req = crate::api::batch::BatchSameRequest {
+            endpoint: Req::ENDPOINT_PATH.to_string(),
+            requests: encoded.collect(),
+        };
+        let responses = self.call(batch_req).await?.responses;
+        let mut decoded = Vec::with_capacity(responses.len());
+        for response in responses {
+            let decoded_msg = <Req as Endpoint>::Response::decode(response.as_ref())?;
+            decoded.push(decoded_msg);
         }
+        Ok(decoded)
+    }
 
-        #[cfg(debug_assertions)]
-        tracing::debug!("Sending request: {:?}", request);
-        let response = handler(&self, request).await;
-        #[cfg(debug_assertions)]
-        tracing::debug!("Received response: {:?}", response);
-
-        response.map_err(Into::into)
+    #[inline(always)]
+    fn auth_status_lock(&self) -> MutexGuard<(AuthStatus, Bytes)> {
+        self.data.auth_status.lock()
     }
 
     /// Get the current auth status.
@@ -268,7 +357,7 @@ impl Client {
     /// # }
     /// ```
     pub fn auth_status(&self) -> AuthStatus {
-        self.auth_status_lock().clone()
+        self.auth_status_lock().0.clone()
     }
 
     /// Get the stored homeserver URL.
@@ -283,7 +372,7 @@ impl Client {
     /// # Ok(())
     /// # }
     /// ```
-    pub fn homeserver_url(&self) -> &Url {
+    pub fn homeserver_url(&self) -> &Uri {
         &self.data.homeserver_url
     }
 
@@ -299,7 +388,7 @@ impl Client {
     /// # Ok(())
     /// # }
     /// ```
-    pub fn make_hmc(&self, id: impl std::fmt::Display) -> Result<Hmc, HmcFromStrError> {
+    pub fn make_hmc(&self, id: impl fmt::Display) -> Result<Hmc, HmcFromStrError> {
         let url = &self.data.homeserver_url;
         Hmc::new(
             format!("{}:{}", url.host().unwrap(), url.port().unwrap()),
@@ -321,8 +410,15 @@ impl Client {
     /// # }
     /// ```
     pub async fn begin_auth(&self) -> ClientResult<()> {
-        let auth_id = api::auth::begin_auth(self, ()).await?.auth_id;
-        *self.auth_status_lock() = AuthStatus::InProgress(auth_id);
+        let auth_id = self
+            .auth()
+            .await
+            .begin_auth(BeginAuthRequest {})
+            .await?
+            .into_message()
+            .await?
+            .auth_id;
+        self.auth_status_lock().0 = AuthStatus::InProgress(auth_id);
         Ok(())
     }
 
@@ -346,18 +442,29 @@ impl Client {
     pub async fn next_auth_step(
         &self,
         response: AuthStepResponse,
-    ) -> ClientResult<Option<AuthStep>> {
+    ) -> ClientResult<Option<NextStepResponse>> {
         if let AuthStatus::InProgress(auth_id) = self.auth_status() {
-            let step = api::auth::next_step(self, AuthResponse::new(auth_id, response)).await?;
+            let step = self
+                .auth()
+                .await
+                .next_step(AuthResponse::new(auth_id, response))
+                .await?
+                .into_message()
+                .await?;
 
-            Ok(if let Some(auth_step::Step::Session(session)) = step.step {
-                *self.data.token_bytes.lock() =
-                    Bytes::copy_from_slice(session.session_token.as_bytes());
-                *self.auth_status_lock() = AuthStatus::Complete(session);
-                None
-            } else {
-                Some(step)
-            })
+            Ok(
+                if let Some(AuthStep {
+                    step: Some(auth_step::Step::Session(session)),
+                    ..
+                }) = step.step
+                {
+                    let token_bytes = Bytes::copy_from_slice(session.session_token.as_bytes());
+                    *self.auth_status_lock() = (AuthStatus::Complete(session), token_bytes);
+                    None
+                } else {
+                    Some(step)
+                },
+            )
         } else {
             Err(ClientError::NoAuthId)
         }
@@ -379,9 +486,15 @@ impl Client {
     /// # Ok(())
     /// # }
     /// ```
-    pub async fn prev_auth_step(&self) -> ClientResult<AuthStep> {
+    pub async fn prev_auth_step(&self) -> ClientResult<StepBackResponse> {
         if let AuthStatus::InProgress(auth_id) = self.auth_status() {
-            api::auth::step_back(self, AuthId::new(auth_id)).await
+            Ok(self
+                .auth()
+                .await
+                .step_back(AuthId::new(auth_id))
+                .await?
+                .into_message()
+                .await?)
         } else {
             Err(ClientError::NoAuthId)
         }
@@ -403,7 +516,7 @@ impl Client {
     /// ```
     pub async fn auth_stream(&self) -> ClientResult<AuthSocket> {
         if let AuthStatus::InProgress(auth_id) = self.auth_status() {
-            let inner = api::auth::stream_steps(self, AuthId::new(auth_id)).await?;
+            let inner = self.auth().await.stream_steps(AuthId::new(auth_id)).await?;
             Ok(AuthSocket { inner })
         } else {
             Err(ClientError::NoAuthId)
@@ -428,8 +541,11 @@ impl Client {
         &self,
         subscriptions: Vec<EventSource>,
     ) -> ClientResult<EventsSocket> {
-        let inner = api::chat::stream_events(self).await?;
-        let mut socket = EventsSocket { inner };
+        let (tx, rx) = self.chat().await.stream_events(()).await?.split();
+        let mut socket = EventsSocket {
+            write: EventsWriteSocket { inner: tx },
+            read: EventsReadSocket { inner: rx },
+        };
         for source in subscriptions {
             socket.add_source(source).await?;
         }
@@ -438,25 +554,92 @@ impl Client {
     }
 }
 
-/// Event subscription socket.
+/// Auth middleware for [`Client`].
 #[derive(Debug, Clone)]
+pub struct AddAuth<S> {
+    inner: S,
+    auth_status: Arc<Mutex<(AuthStatus, Bytes)>>,
+}
+
+impl<S> Service<TransportRequest> for AddAuth<S>
+where
+    S: Service<TransportRequest, Response = TransportResponse>,
+{
+    type Response = S::Response;
+
+    type Error = S::Error;
+
+    type Future = S::Future;
+
+    fn poll_ready(
+        &mut self,
+        cx: &mut std::task::Context<'_>,
+    ) -> std::task::Poll<Result<(), Self::Error>> {
+        Service::poll_ready(&mut self.inner, cx)
+    }
+
+    fn call(&mut self, mut req: TransportRequest) -> Self::Future {
+        let guard = self.auth_status.lock();
+        if guard.0.is_authenticated() {
+            let req = match &mut req {
+                TransportRequest::Unary(req) => req,
+                TransportRequest::Socket(req) => req,
+            };
+            req.get_or_insert_header_map().insert(
+                http::header::AUTHORIZATION,
+                // This is safe on the assumption that servers will never send session tokens
+                // with invalid-byte(s). If they do, they aren't respecting the protocol
+                unsafe { http::HeaderValue::from_maybe_shared_unchecked(guard.1.clone()) },
+            );
+        }
+
+        Service::call(&mut self.inner, req)
+    }
+}
+
+/// Event subscription socket.
 pub struct EventsSocket {
-    inner: hrpc::client::socket::Socket<
-        crate::api::chat::StreamEventsRequest,
-        crate::api::chat::Event,
-    >,
+    read: EventsReadSocket,
+    write: EventsWriteSocket,
 }
 
 impl EventsSocket {
     /// Get an event.
-    pub async fn get_event(&mut self) -> Option<ClientResult<crate::api::chat::event::Event>> {
-        let res = self.inner.get_message().await?;
-        match res {
-            Ok(ev) => ev.event.map(Ok),
-            Err(err) => Some(Err(err.into())),
-        }
+    #[inline]
+    pub async fn get_event(&mut self) -> ClientResult<Option<crate::api::chat::Event>> {
+        self.read.get_event().await
     }
 
+    /// Add a new event source.
+    #[inline]
+    pub async fn add_source(&mut self, source: EventSource) -> ClientResult<()> {
+        self.write.add_source(source).await
+    }
+
+    /// Close this socket.
+    #[inline]
+    pub async fn close(self) -> ClientResult<()> {
+        self.write.close().await
+    }
+
+    /// Split this socket into the read and write half.
+    pub fn split(self) -> (EventsWriteSocket, EventsReadSocket) {
+        (self.write, self.read)
+    }
+}
+
+impl Debug for EventsSocket {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        f.write_str("EventsSocket")
+    }
+}
+
+/// Write half of the event subscription socket.
+pub struct EventsWriteSocket {
+    inner: hrpc::client::socket::WriteSocket<crate::api::chat::StreamEventsRequest>,
+}
+
+impl EventsWriteSocket {
     /// Add a new event source.
     pub async fn add_source(&mut self, source: EventSource) -> ClientResult<()> {
         self.inner
@@ -471,20 +654,49 @@ impl EventsSocket {
     }
 }
 
+impl Debug for EventsWriteSocket {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        f.write_str("EventsWriteSocket")
+    }
+}
+
+/// Read half of the event subscription socket.
+pub struct EventsReadSocket {
+    inner: hrpc::client::socket::ReadSocket<crate::api::chat::StreamEventsResponse>,
+}
+
+impl EventsReadSocket {
+    /// Get an event.
+    pub async fn get_event(&mut self) -> ClientResult<Option<crate::api::chat::Event>> {
+        let resp = self.inner.receive_message().await?;
+        Ok(resp
+            .event
+            .and_then(|a| crate::api::chat::Event::try_from(a).ok()))
+    }
+}
+
+impl Debug for EventsReadSocket {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        f.write_str("EventsReadSocket")
+    }
+}
+
 /// Auth steps subscription socket.
-#[derive(Debug, Clone)]
 pub struct AuthSocket {
-    inner: hrpc::client::socket::ReadSocket<
+    inner: hrpc::client::socket::Socket<
         crate::api::auth::StreamStepsRequest,
-        crate::api::auth::AuthStep,
+        crate::api::auth::StreamStepsResponse,
     >,
 }
 
 impl AuthSocket {
     /// Get an auth step.
-    pub async fn get_step(&mut self) -> Option<ClientResult<crate::api::auth::AuthStep>> {
-        let res = self.inner.get_message().await?;
-        Some(res.map_err(Into::into))
+    pub async fn get_step(&mut self) -> ClientResult<Option<crate::api::auth::AuthStep>> {
+        self.inner
+            .receive_message()
+            .await
+            .map(|s| s.step)
+            .map_err(Into::into)
     }
 
     /// Close this socket.
@@ -492,3 +704,9 @@ impl AuthSocket {
         self.inner.close().await.map_err(Into::into)
     }
 }
+
+impl Debug for AuthSocket {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        f.write_str("AuthSocket")
+    }
+}
